;=================================================
;   program overvector
;   F. Heitsch, Jun 1999
;=================================================

pro overvector,contdata,field1,field2,data,nx,ny,frac,scale,$
               color      = color,$
               cont       = cont,$
               invert     = invert,$
;               log        = log,$
               print      = print,$
;               gif        = gif,$
;               stretch    = stretch,$
               nohead     = nohead,$
;               aspect     = aspect,$
               threshdata = threshdata,$
               vector     = vector,$
               cutedge    = cutedge,$
               ifnm       = ifnm,$
               dminmax    = dminmax,$
               cminmax    = cminmax,$
               markbox    = markbox,$
               histbox    = histbox,$
               nosmooth   = nosmooth,$
               clabels    = clabels,$
               winsize    = winsize,$
               legend     = legend,$
               minmaxsurf = minmaxsurf,$
               clegend    = clegend


;=================================================
; PARAMETERS
;
;   contdata    : data for contours, plotted if keyword cont is set.
;   field1      : x-component for vectors, plotted if keyword vector is set
;   field2      : y-component for vectors, plotted if keyword vector is set
;   data        : data for surface plot
;   nx, ny      : x (and y) dimensions of previous four 2D-arrays
;   frac        : fraction of vectors to be plotted.1=every cell, 2 every second
;   scale       : scale of vectors: default: 1.0 = one grid cell length
;
; KEYWORDS
;
;   color       : if set, loadct,34 instead of loadct,0
;   cont        : overplot contours. Must contain number of contours.
;   invert      : invert color table
;   log         : plot vector length logarithmically
;   print       : output to ps-file with name 'overvector.ps' or ifnm if set.
;   gif         : makes gif-dump 
;   stretch     : stretch color table
;   nohead      : don't plot arrow heads (polarization maps)
;   ny          : gives y-dimension in case nx .ne. ny
;   aspect      : for handling non-quadratic fields
;   threshdata  : determines, above which value of data the vector color will be
;                 switched from maxcolor to mincolor in order to make the 
;                 vectors visible for high values of data. Also applies to contours.
;                 Note that the color switch is triggered by data, not contdata.
;   vector      : overplot vectors
;   cutedge     : prevents overspilling of vectors
;   ifnm        : filename for output file in case print is set
;   dminmax     : minimum and maximum data value (used for scaling)
;   cminmax     : minimum and maximum contdata value (used for scaling)
;   markbox     : vectors [x1,y1,x2,y2] denoting the lower left and
;                 upper right corner of a rectangular box to be
;                 plotted on the frame. If several boxes are to be
;                 plotted, append the next set of coordinates.
;                 [xa1,...,ya2,xb1,...,yb2,...]
;   histbox     : if set to a vector of the format given below,
;                 overvector will overplot greydata with histograms
;                 of the quantity given in histarr (see format).
;                 The user has to make sure that the spatial information
;                 given in the histogram corresponds to the greydata plot.
;                 Format of the vector:
;                 histbox = [n1, n2, subsize, histarr], where
;                      n1      = x-dimension in pixel (float)
;                      n2      = y-dimension in pixel (float)
;                      subsize = size of histogram box. n1,n2 must be multiples
;                                of subsize.
;                      bins    = number of bins in histogram
;                      histarr = fltarr(bins,2,n1/subsize,n2/subsize), where
;                                the second index = 0: x-ordinate
;                                the second index = 1: y-ordinate (count rate)
;                                The data have to be stored consecutively.
;   nosmooth    : Prevents interpolation on grid in screen mode (for debugging).
;   clabels     : structure consisting of fields
;                 ca: array with annotations for contours (c_annotation)
;                 cw: array determining which contours to be plotted (c_labels)

;
; USE:
;
;   For overplotting vectors, overvector uses velvec.pro, a modified version
;   of velovect.pro. Vectors not to be plotted have to be indicated by
;   setting the corresponding field1 and field2 values to > 1.0e08.
;
;=======================================================

  debug      = 1       ; formerly keyword. Now default.

  if keyword_set(minmaxsurf) then xbaroff = 0.05 else xbaroff = 0.00
  xbaroff = 0.0

  if (not keyword_set(print)) then print = 'ps'

  if keyword_set(winsize) then begin
    sizewinx   = winsize[0]; size of display window
    sizewiny   = winsize[1]
  endif else begin
    sizewinx = 512 + fix(xbaroff*512.0)
    sizewiny = 512
  endelse
  xdevsize   = 15.0 + xbaroff*15.0    ; device size (ps) in centimeters
  ydevsize   = 15.0

  if keyword_set(cont) then begin
    contlevels = cont       ; contour levels
  endif else begin
    contlevels = 8
  endelse

  ;adapting length of arrows according to frac
  if frac ne 1 then begin
    len = scale*double(frac)/sqrt(2.0)
  endif else begin
    len = scale
  endelse

  n1 = nx
  n2 = ny
  if (n1 gt n2) then begin
    sizewiny = fix(float(sizewinx)*float(n2)/float(n1))
    ydevsize = xdevsize*float(n2)/float(n1)
  endif
  if (n1 lt n2) then begin
    sizewinx = fix(float(sizewiny)*float(n1)/float(n2))
    xdevsize = ydevsize*float(n1)/float(n2)
  endif

  ; now scale the surface data
  if keyword_set(dminmax) then begin
    if (print eq 'ps') then begin
      if keyword_set(nosmooth) then begin
        img  = bytscl(congrid(data,sizewinx,sizewiny),max=dminmax(1),min=dminmax(0))
        print,'OVERVECTOR: no interpolation'
      endif else begin
        img  = bytscl(congrid(data,sizewinx,sizewiny,/interp),max=dminmax(1),min=dminmax(0))
      endelse
    endif else begin
      img = bytscl(data,max=dminmax(1),min=dminmax(0))
    endelse
  endif else begin
    if (print eq 'ps') then begin    
      if keyword_set(nosmooth) then begin
        img  = bytscl(congrid(data,sizewinx,sizewiny))
        print,'OVERVECTOR: no interpolation'
      endif else begin
        img  = bytscl(congrid(data,sizewinx,sizewiny,/interp))
      endelse
    endif else begin 
      img  = bytscl(congrid(data,sizewinx,sizewiny,/interp))
    endelse
  endelse

  if keyword_set(cminmax) then begin
    cmin = cminmax[0]
    cmax = cminmax[1]
  endif else begin
    cmin = min(contdata)
    cmax = max(contdata)
  endelse
  if keyword_set(debug) then begin
    if keyword_set(dminmax) then begin
      print,'OVERVECTOR: dmin          = ',dminmax(0)
      print,'OVERVECTOR: dmax          = ',dminmax(1)
    endif
    print,'OVERVECTOR: min(data)     = ',min(data)
    print,'OVERVECTOR: max(data)     = ',max(data)
    print,'OVERVECTOR: min(contdata) = ',cmin
    print,'OVERVECTOR: max(contdata) = ',cmax
  endif

  ;switch the device
  if keyword_set(print) then begin
    if (print eq 'ps') then begin
      set_plot,'ps'
      if keyword_set(ifnm) then fnm=ifnm+'.ps' else fnm='overvector.ps'
      device,filename=fnm,xsize=xdevsize,ysize=ydevsize,/portrait,$
          /color,bits=8,xoffset=3.0,yoffset=3.0
    endif else begin
      if (print eq 'eps') then begin
        set_plot,'ps'
        if keyword_set(ifnm) then fnm=ifnm+'.eps' else fnm='overvector.eps'
        device,filename=fnm,xsize=xdevsize,ysize=ydevsize,/portrait,$
            /color,bits=8,xoffset=3.0,yoffset=3.0,/encaps
      endif else begin
        if (print eq 'tif') then begin
;          set_plot,'Z',/interpolate
;          device,set_resolution=[sizewinx,sizewiny]
          set_plot,'x'
          window,xsize=sizewinx,ysize=sizewiny
        endif else begin
          if (print eq 'png') then begin
            set_plot,'Z',/interpolate
            device,set_resolution=[sizewinx,sizewiny]
          endif else begin
            set_plot='x'
            window,xsize=sizewinx,ysize=sizewiny
          endelse
        endelse
      endelse
    endelse
  endif else begin
    print,'OVERVECOR: print keyword undefined.'
    retall
  endelse

  ;save the old values and reset them at the end
  xm = !x.margin
  ym = !y.margin
  !p.multi  = 0
  !x.margin = 0
  !y.margin = 0

  xxxxx = findgen(n1)/n1*2.0-1.0
  yyyyy = findgen(n2)/n2*2.0-1.0

  ; this is necessary to set the plot window correctly
  plot,[0.0,0.0],[0.0,0.0],/nodata,position=[0.0,0.0,1.0,1.0],$
       /noerase,xstyle=4,ystyle=4

  px = !x.window * !d.x_vsize
  py = !y.window * !d.y_vsize
  swx = px[1] - px[0]            ;Size in x in device units
  swy = py[1] - py[0]            ;Size in Y
  if keyword_set(debug) then begin
    print,'OVERVECTOR: !d.x_vsize, !d.y_vsize = ',!d.x_vsize,!d.y_vsize
    print,'OVERVECTOR: !x.window              = ',!x.window
    print,'OVERVECTOR: !y.window              = ',!y.window
    print,'OVERVECTOR: px                     = ',px[0],px[1]
    print,'OVERVECTOR: py                     = ',py[0],py[1]
    print,'OVERVECTOR: swx,swy                = ',swx,swy
  endif
  six = float(n1)
  siy = float(n2)
  aspi= six/siy
  aspw= swx/swy
  f   = aspi/aspw

  if f ge 1.0 then begin
    if keyword_set(aspect) then begin
      swy = swy / f 
      sizewiny = sizewiny / f
      ydevsize = ydevsize / f
    endif
  endif else begin
    if keyword_set(aspect) then begin
      swx = swx * f
      sizewinx = sizewinx * f
      xdevsize = xdevsize * f
    endif
  endelse

  contour,[[0.0+xbaroff,0.0],[1.0,1.0]],/nodata, xstyle=4,ystyle=4,title=title

  if keyword_set(color) then begin
    loadct,color  ;34
  endif else begin
    loadct,0
  endelse
;  if keyword_set(stretch) then stretch,10,250,1.40 ;1.20
  if keyword_set(invert)  then begin
    invertct,255,0,debug=debug
    mincolor = 0;255
  endif else begin
    mincolor = 0
  endelse

  maxcolor = !d.n_colors-1

  if keyword_set(debug) then print,'  maxcolor = ',maxcolor
  colors = intarr(contlevels)
  if keyword_set(threshdata) then begin
    for i=0,contlevels-1 do begin
      tempaa = (cmax-cmin)*float(i)/float(contlevels) + cmin
      if (tempaa ge threshdata) then begin
        colors[i] = mincolor
      endif else begin
        colors[i] = maxcolor
      endelse
    endfor
    print,colors
  endif else begin
    colors[*] = maxcolor
  endelse

  rgbred = intarr(256)
  rgbblu = intarr(256)
  rgbgrn = intarr(256)
  tvlct,rgbred,rgbgrn,rgbblu,/get
  
  if (print eq 'ps') then begin
    tv,img,px[0]*(1.0+xbaroff),py[0],xsize=swx,ysize=swy,/device
  endif else begin
    if keyword_set(nosmooth) then begin
      tvscl,congrid(img,swx,swy)
    endif else begin
      tv,poly_2d(img,$  ;Have to resample image
                 [[0,0],[six/swx,0]], [[0,siy/swy],[0,0]],$
                 1,swx,swy), $
         px[0],py[0]
    endelse
  endelse                       ;scalable pixels

  if keyword_set(cont) then begin
    if keyword_set(clabels) then begin
      follow=1
      downhill=0
      c_annotation=clabels.ca
      c_labels    =clabels.cw
    endif else begin
      c_annotation = ''
      c_labels     = 0
    endelse
    ; careful. We allocate 2 additional levels here (the first and last one).
    ; They will not be plotted. contlevels is the number of levels actually 
    ; plotted.
    levelarr=findgen(contlevels+2)*(cmax-cmin)/float(contlevels+1)+cmin
    levelarr=levelarr[1:contlevels]
    levelnum=(size(levelarr,/dimensions))[0]
    for lev=0,levelnum-1 do begin
      print,format='("OVERVECTOR: level[",i2,"] = ",e12.5)',lev,levelarr[lev]
    endfor
    contour,contdata,xxxxx+1.0/float(n1),yyyyy+1.0/float(n2),/noerase $
            ,c_charsize=1.0 $
            ,levels=levelarr $
            ,c_color=colors  $
            ,thick=3  $
            ,follow = follow $
            ,c_labels = c_labels $
            ,c_annotation = c_annotation $
            ,downhill = downhill $
            ,title=title $
            ,charsize=2.2  $
            ,xstyle=4,ystyle=4 $
            ,xrange=[-1.0,1.0],yrange=[-1.0,1.0],color=maxcolor
            
  endif

  if keyword_set(log) then begin
    field1 = sign(field1)*alog10(abs(field1))
    field2 = sign(field2)*alog10(abs(field2))
  endif

  if keyword_set(debug) then begin
    print,'OVERVECTOR: min(field1^2+field2^2) = ',min(field1^2+field2^2)
    print,'OVERVECTOR: max(field1^2+field2^2) = ',max(field1^2+field2^2)
  endif

  if keyword_set(vector) then begin
    if keyword_set(threshdata) then begin
      if keyword_set(dminmax) then begin
        invmask = data ge 0.5*(dminmax[0]+dminmax[1])
      endif else begin
        invmask = data ge threshdata
      endelse
      if keyword_set(debug) then begin
        print,'OVERVECTOR: total of inverted colors: ',total(invmask)
      endif
    endif
    velvec,refsel(field1,n1,ny=n2,frac,cutedge=cutedge,debug=debug),$
           refsel(field2,n1,ny=n2,frac,cutedge=cutedge,debug=debug),$
           /noerase,$
           xtickname=[' ',' ',' ',' ',' ',' ',' '],$
           ytickname=[' ',' ',' ',' ',' ',' ',' '],$
           dots=0,length=len,missing=1.0e8,/device,charsize=1.2,$
           color=maxcolor,nohead=nohead,invmask=invmask,mincolor=mincolor,$
           debug=debug,thick=3
  endif

  if keyword_set(markbox) then begin
    print,'OVERVECTOR: overplotting boxes'
    coordnum = (size(markbox))(1)
    if (coordnum mod 4 ne 0) then begin
      print,'OVERVECTOR: error: invalid coordinate number: ',coordnum
      print,'                   Will skip markbox plotting.'
    endif else begin
      coordnum = coordnum / 4
      for i=0,coordnum-1 do begin
        if keyword_set(threshdata) then begin
          meandata = (   data(markbox(i*4+0),markbox(i*4+1)) $
                       + data(markbox(i*4+2),markbox(i*4+1)) $
                       + data(markbox(i*4+2),markbox(i*4+3)) $
                       + data(markbox(i*4+2),markbox(i*4+3))  ) * 0.25
          if keyword_set(debug) then begin
            print,'OVERVECTOR: meandata = ',meandata
          endif
          if (meandata ge threshdata) then begin
            markcolor = mincolor
          endif else begin
            markcolor = maxcolor
          endelse
        endif else begin
          markcolor = maxcolor
        endelse
        print,'OVERVECTOR: marking boxes in color ',markcolor

;        plots,[markbox[i*4+0]/,markbox[i*4+2]],$
;              [markbox[i*4+1],markbox[i*4+1]],$
;              thick=3,color=markcolor,/device
;        plots,[markbox[i*4+0],markbox[i*4+2]],$
;              [markbox[i*4+3],markbox[i*4+3]],$
;              thick=3,color=markcolor,/device
;        plots,[markbox[i*4+0],markbox[i*4+0]],$
;              [markbox[i*4+1],markbox[i*4+3]],$
;              thick=3,color=markcolor,/device
;        plots,[markbox[i*4+2],markbox[i*4+2]],$
;              [markbox[i*4+1],markbox[i*4+3]],$
;              thick=3,color=markcolor,/device

        plots,[px[0]+swx*float(markbox[i*4+0])/float(n1),py[0]+swy*float(markbox[i*4+2])/float(n2)],$
              [px[0]+swx*float(markbox[i*4+1])/float(n1),py[0]+swy*float(markbox[i*4+1])/float(n2)],$
              thick=3,color=markcolor,/device
        plots,[px[0]+swx*float(markbox[i*4+0])/float(n1),py[0]+swy*float(markbox[i*4+2])/float(n2)],$
              [px[0]+swx*float(markbox[i*4+3])/float(n1),py[0]+swy*float(markbox[i*4+3])/float(n2)],$
              thick=3,color=markcolor,/device
        plots,[px[0]+swx*float(markbox[i*4+0])/float(n1),py[0]+swy*float(markbox[i*4+0])/float(n2)],$
              [px[0]+swx*float(markbox[i*4+1])/float(n1),py[0]+swy*float(markbox[i*4+3])/float(n2)],$
              thick=3,color=markcolor,/device
        plots,[px[0]+swx*float(markbox[i*4+2])/float(n1),py[0]+swy*float(markbox[i*4+2])/float(n2)],$
              [px[0]+swx*float(markbox[i*4+1])/float(n1),py[0]+swy*float(markbox[i*4+3])/float(n2)],$
              thick=3,color=markcolor,/device
        xyouts,px[0]+1.01*swx*float(markbox[i*4+0])/float(n1),py[0]+1.01*swy*float(markbox[i*4+1])/float(n2),$
               '!6'+string(i+1,format='(i2)'),charsize=1.0,color=markcolor,/device
      endfor
    endelse      
        
  endif

  if keyword_set(legend) then begin
    if (n_elements(clegend) eq 2) then begin
      chsz=clegend[0]
      markcolor=fix(clegend[1])
    endif else begin
      chsz = 1.0
      markcolor=mincolor
    endelse
    xyouts,px[0]+0.04*swx,py[0]+0.04*swy,$
          '!6'+legend,color=markcolor,/device,charsize=chsz,charthick=3.0
  endif

  if keyword_set(minmaxsurf) then begin
    minsurf = minmaxsurf[0]
    maxsurf = minmaxsurf[1]
    if (minsurf lt maxsurf) then begin
  
;      colorbar,min=minsurf,max=maxsurf,format='(e8.1)',/vertical,ncolors=256,title=legbar,$
;               charsize=0.8,position=[0.00,0.50,0.05,0.95],color=0,/pscolor
      colorbar,min=minsurf,max=maxsurf,format='(f4.1)',/vertical,ncolors=256,title=legbar,$
               charsize=chsz,charthick=3.0,position=[0.00,0.50,0.05,0.95],color=markcolor,/pscolor,/right
  
    endif
  endif


  if keyword_set(histbox) then begin
    hn1     = fix(histbox(0))
    hn2     = fix(histbox(1))
    subsize = fix(histbox(2))
    bins    = fix(histbox(3))

    if keyword_set(debug) then begin
      print,'OVERVECTOR: hn2     = ',hn1
      print,'OVERVECTOR: hn1     = ',hn2
      print,'OVERVECTOR: subsize = ',subsize
      print,'OVERVECTOR: bins    = ',bins
    endif

    if ((hn1 mod subsize ne 0) or (hn2 mod subsize ne 0)) then begin
      print,'OVERVECTOR: error: hn mod subsize ne 0'
      print,'                   Will skip histbox plotting.'   
    endif else begin
      imod = hn1 / subsize
      jmod = hn2 / subsize

      temparr = histbox(4:n_elements(histbox)-1)
      histarr = fltarr(bins,2,imod,jmod)
      for l=0l,jmod-1 do begin
        for k=0l,imod-1 do begin
          for j=0l,1 do begin
            for i=0l,bins-1 do begin
              histarr(i,j,k,l) = temparr(i*2*imod*jmod+j*imod*jmod+k*jmod+l)
            endfor
          endfor
        endfor
      endfor

      tempaa = 1.0 / float(imod)
      tempab = 1.0 / float(jmod)
      minvlos = min(histarr(*,0,*,*))
      maxvlos = max(histarr(*,0,*,*))
      minhist = min(histarr(*,1,*,*))
      maxhist = max(histarr(*,1,*,*))

      if keyword_set(debug) then begin
        print,'OVERVECTOR: minvlos = ',minvlos
        print,'OVERVECTOR: maxvlos = ',maxvlos
        print,'OVERVECTOR: minvlos = ',minhist
        print,'OVERVECTOR: maxvlos = ',maxhist
      endif
   
      for j=0,jmod-1 do begin
        for i=0,imod-1 do begin
          if keyword_set(threshdata) then begin
            meandata = mean(data(i*subsize:(i+1)*subsize-1,$
                                 j*subsize:(j+1)*subsize-1))

            if keyword_set(debug) then begin
              print,'OVERVECTOR: meandata = ',meandata
            endif
            if (meandata ge threshdata) then begin
              markcolor = mincolor
            endif else begin
              markcolor = maxcolor
            endelse
          endif
          plot,histarr(*,0,i,j),histarr(*,1,i,j),$
               xrange=[minvlos,maxvlos],yrange=[minhist,maxhist],$
               xcharsize=0.00001,ycharsize=0.00001,$
               ;xstyle=4,ystyle=4,$
               ;xticks=1,yticks=1,$
               ;xticklen=0.0,yticklen=0.0,$
               position=[i*tempaa,j*tempab,(i+1)*tempaa,(j+1)*tempab],$
               /noerase,color=markcolor
        endfor
      endfor  
;      plot,histarr(*,0,0,0),histarr(*,1,0,0),/nodata,$
;           position=[0.0,0.0,1.0,1.0]
      !p.noerase=0
    endelse
  endif

  if (print eq 'tif') then begin
    if keyword_set(ifnm) then fnm = ifnm+'.tif' else fnm = 'overvector.tif'
    write_tiff,fnm,tvrd(/true)
  endif

  if (print eq 'png') then begin
    if keyword_set(ifnm) then fnm = ifnm+'.png' else fnm = 'overvector.png'
    bytimg = tvrd()
    tmpimg = bytarr(3,(size(bytimg,/dim))[0],(size(bytimg,/dim))[1])
    tmpimg[0,*,*] = rgbred[bytimg]
    tmpimg[1,*,*] = rgbgrn[bytimg]
    tmpimg[2,*,*] = rgbblu[bytimg]
    write_png,fnm,tmpimg,rgbred,rgbgrn,rgbblu
  endif

  if ((print eq 'ps') or (print eq 'png')) then begin
    device,/close
    set_plot,'x'
  endif
  !x.margin = xm
  !y.margin = ym

end




















