; $Id: velvec.pro,v 2.0 1999/11/29 18:55:49 $
;
; Copyright (c) 1983-1998, Research Systems, Inc.  All rights reserved.
;	Unauthorized reproduction prohibited.
;
; modified 1: Jul 1999: F. Heitsch
;             included  keyword nohead
;             increased number of plotable arrows
; modified 2: Dec 2000: F. Heitsch 
;             included keyword invmask


pro velvec,u,v,x,y,$
           missing = missing,$	
           length  = length,$
           dots    = dots,$
           color   = color,$
           clip    = clip,$
           noclip  = noclip,$
           _EXTRA  = extra,$
           nohead  = nohead,$
           invmask = invmask,$
           mincolor= mincolor,$
           debug   = debug
;
;+ 
; NAME:
;	VELOVECT
;
; PURPOSE:
;	Produce a two-dimensional velocity field plot.
;
;	A directed arrow is drawn at each point showing the direction and 
;	magnitude of the field.
;               
; CATEGORY:
;	Plotting, two-dimensional.
;
; CALLING SEQUENCE:
;	VELOVECT, U, V [, X, Y]
;
; INPUTS:
;	U:	The X component of the two-dimensional field.  
;		U must be a two-dimensional array.
;
;	V:	The Y component of the two dimensional field.  Y must have
;		the same dimensions as X.  The vector at point [i,j] has a 
;		magnitude of:
;
;			(U[i,j]^2 + V[i,j]^2)^0.5
;
;		and a direction of:
;
;			ATAN2(V[i,j],U[i,j]).
;
; OPTIONAL INPUT PARAMETERS:
; 	X:	Optional abcissae values.  X must be a vector with a length 
;		equal to the first dimension of U and V.
;
;	Y:	Optional ordinate values.  Y must be a vector with a length
;		equal to the first dimension of U and V.
;
; KEYWORD INPUT PARAMETERS:
;      MISSING:	Missing data value.  Vectors with a LENGTH greater
;		than MISSING are ignored.
;
;	LENGTH:	Length factor.  The default of 1.0 makes the longest (U,V)
;		vector the length of a cell.
;
;	DOTS:	Set this keyword to 1 to place a dot at each missing point. 
;		Set this keyword to 0 or omit it to draw nothing for missing
;		points.  Has effect only if MISSING is specified.
;
;	COLOR:	The color index used for the plot.
;
;       nohead: No arrow heads will be drawn
;
;       invmask:logical mask. If element is true, color of arrow will be inverted
;
;       mincolor: color used for invmask .eq. true
;               
;       debug   : debugging statements are printed.
;
;	Note:   All other keywords are passed directly to the PLOT procedure
;		and may be used to set option such as TITLE, POSITION, 
;		NOERASE, etc.
; OUTPUTS:
;	None.
;
; COMMON BLOCKS:
;	None.
;
; SIDE EFFECTS:
;	Plotting on the selected device is performed.  System
;	variables concerning plotting are changed.
;
; RESTRICTIONS:
;	None.
;
; PROCEDURE:
;	Straightforward.  Unrecognized keywords are passed to the PLOT
;	procedure.  
;
; MODIFICATION HISTORY:
;	DMS, RSI, Oct., 1983.
;	For Sun, DMS, RSI, April, 1989.
;	Added TITLE, Oct, 1990.
;	Added POSITION, NOERASE, COLOR, Feb 91, RES.
;	August, 1993.  Vince Patrick, Adv. Visualization Lab, U. of Maryland,
;		fixed errors in math.
;	August, 1993. DMS, Added _EXTRA keyword inheritance.
;	January, 1994, KDB. Fixed integer math which produced 0 and caused
;		            divide by zero errors.
;	December, 1994, MWR. Added _EXTRA inheritance for PLOTS and OPLOT.
;	June, 1995, MWR. Removed _EXTRA inheritance for PLOTS and changed
;			 OPLOT to PLOTS.
;       September, 1996, GGS. Changed denominator of x_step and y_step vars. 
;       February, 1998, DLD.  Add support for CLIP and NO_CLIP keywords.
;       November, 1999, Fabian Heitsch, switching of arrows
;
  on_error,2                      ;Return to caller if an error occurs
  s = size(u)
  t = size(v)
  if (( s[0] ne 2) or (total(abs(s[0:2]-t[0:2])) ne 0)) then begin 
    print,'VELVEC: error: U and V parameters must be 2D and same size.'
    return
  endif

  if n_params(0) lt 3 then begin
    x = findgen(s[1]) 
  endif else begin
    if n_elements(x) ne s[1] then begin
      print,'VELVEC: error: X and Y arrays have incorrect size.'
      return
    endif
  endelse
  if n_params(1) lt 4 then begin
    y = findgen(s[2]) 
  endif else begin
    if n_elements(y) ne s[2] then begin
      print,'VELVEC: error: X and Y arrays have incorrect size.'
      return
    endif
  endelse

  if n_elements(missing) le 0 then missing = 1.0e30
  if n_elements(length) le 0 then length = 1.0

  mag = sqrt(u^2.+v^2.)             ;magnitude.
  ;Subscripts of good elements
  nbad = 0l                        ;# of missing points
  ngood= 0l
  if n_elements(missing) gt 0 then begin
    good = where(mag lt missing,ngood) 
    if keyword_set(dots) then bad = where(mag ge missing, nbad)
  endif else begin
    ngood= n_elements(mag) 
    good = lindgen(ngood)
  endelse

  if (ngood eq 0l) then begin
    print,'OVERVECTOR: error: no valid vector points.'
  endif else begin
    ugood   = u[good]
    vgood   = v[good]
    if keyword_set(invmask) then begin
      if not keyword_set(mincolor) then colormin = 0 else colormin = mincolor
      if keyword_set(debug) then print,'VELVEC: colormin = ',colormin
    endif
    normcolor = color

    x0 = min(x)                     ;get scaling
    x1 = max(x)
    y0 = min(y)
    y1 = max(y)
    x_step=(x1-x0)/(s[1]-1.0)   ; Convert to float. Integer math
    y_step=(y1-y0)/(s[2]-1.0)   ; could result in divide by 0

    maxmag=max([max(abs(ugood/x_step)),max(abs(vgood/y_step))])
    sina = length * (ugood/maxmag)
    cosa = length * (vgood/maxmag)

    if n_elements(title) le 0 then title = ''
    ;--------------  plot to get axes  ---------------
    if n_elements(color) eq 0 then color = !p.color
    if n_elements(noclip) eq 0 then noclip = 1
    x_b0=x0-x_step
    x_b1=x1+x_step
    y_b0=y0-y_step
    y_b1=y1+y_step
    if n_elements(position) eq 0 then begin
      plot,[x_b0,x_b1],[y_b1,y_b0],/nodata,/xst,/yst, $
           color=color, _EXTRA = extra
    endif else begin
      plot,[x_b0,x_b1],[y_b1,y_b0],/nodata,/xst,/yst, $
          color=color, _EXTRA = extra
    endelse
    if n_elements(clip) eq 0 then $
      clip = [!x.crange[0],!y.crange[0],!x.crange[1],!y.crange[1]]

    if keyword_set(nohead) then begin
      st = 0.0
      ct = 0.0
    endif else begin
      r     = .3                          ;len of arrow head
      angle = 22.5 * !dtor            ;Angle of arrowhead
      st    = r * sin(angle)             ;sin 22.5 degs * length of head
      ct    = r * cos(angle)
    endelse

    for i=0l,n_elements(good)-1 do begin     ;Each point
      if keyword_set(invmask) then begin
        if (invmask[good[i]] eq 1) then begin
          color = colormin
        endif else begin
          color = normcolor
        endelse
      endif
      x0 = x[good[i] mod s[1]]        ;get coords of start & end
      dx = sina[i]
      x1 = x0 + dx
      y0 = y[good[i] / s[1]]
      dy = cosa[i]
      y1 = y0 + dy
      xd = x_step
      yd = y_step
      plots,[x0,x1,x1-(ct*dx/xd-st*dy/yd)*xd, $
    	     x1,x1-(ct*dx/xd+st*dy/yd)*xd], $
            [y0,y1,y1-(ct*dy/yd+st*dx/xd)*yd, $
	     y1,y1-(ct*dy/yd-st*dx/xd)*yd], $
            color=color,clip=clip,noclip=noclip,thick=3;thick
    endfor
    if nbad gt 0 then begin             ;Dots for missing?
      PLOTS, x[bad mod s[1]], y[bad / s[1]], psym=3, color=color, $
             clip=clip,noclip=noclip
    endif
  endelse

;===========================================================

end





